Package: ratelimitr
Type: Package
Title: Rate Limiting for R
Version: 0.4.2
Authors@R: person(given = "Tarak",
                  family = "Shah",
                  role = c("aut", "cre", "cph"),
                  email = "tarak.shah@gmail.com")
Description: Allows to limit the rate at which one or more functions can be called.
License: MIT + file LICENSE
RoxygenNote: 6.1.0
Suggests: testthat, microbenchmark, knitr, rmarkdown, covr
Imports: assertthat
VignetteBuilder: knitr
URL: https://github.com/tarakc02/ratelimitr
BugReports: https://github.com/tarakc02/ratelimitr/issues
NeedsCompilation: no
Packaged: 2025-10-12 20:26:22 UTC; tshah
Author: Tarak Shah [aut, cre, cph]
Maintainer: Tarak Shah <tarak.shah@gmail.com>
Repository: CRAN
Date/Publication: 2025-10-13 13:40:02 UTC
Built: R 4.6.0; ; 2026-04-25 16:50:27 UTC; unix
