# Generated by roxygen2: do not edit by hand

S3method(get_function,rate_limited_function)
S3method(get_precision,limited_function_list)
S3method(get_precision,rate_limited_function)
S3method(get_rates,limited_function_list)
S3method(get_rates,rate_limited_function)
S3method(limit_rate,"function")
S3method(limit_rate,function_list)
S3method(limit_rate,list)
S3method(print,limited_function_list)
S3method(print,rate_limited_function)
S3method(reset,limited_function_list)
S3method(reset,rate_limited_function)
export(UPDATE_RATE)
export(get_function)
export(get_precision)
export(get_rates)
export(limit_rate)
export(rate)
export(reset)
import(assertthat)
